/* Figure 2 - all panels */

// Select panels of Fig 2 to be generated (1=abc, 2=def, 3=ghi)
#define Panel 1

// Constants setting the simulations in the window 
#define L_INIT 0.1	// initial length of the filament
#define X_SCALE 2.5	// scaling factor for interactively defined functions of positions.
#define HEIGHT 2	// total height of the plot
#define dt 1		// time step
#define dhdt 0.0013	// vertical displacement of the filament per unit time.
#define STEPS (HEIGHT / (dhdt*dt)-1) // number of simulation steps

// Function approximating positional information propagating by diffusion
// (it would be exact for a semi-infinite medium in a quasi-static static state)
// For visualization only: the actual simulation propagates distances directly.
#define e 2.718		// approx. natural logarithm base
#define MorphConc(x) 0.5*e^(-(x))	// morphogen concentration as a function of position

// Definition of parameters and functions for each Panel
#if (Panel==1)
// Figure 2abc: constant growth, constant threshold
#define GrowthFunc(x) 0.001955 	// constant growth rate
#define DivThreshold(x) 0.15	// constrant cell division threshold
#define L_FIN  2.03			// anticipated final filament length
#define TH_RW  0.1925		// color threshold 1: red to white
#define TH_WB  0.5785		// color threshold 2: white to blue
#endif

#if (Panel==2)
// Figure 2def: constant growth, smoothly increasing threshold
#define GrowthFunc(x) 0.001955 			// constant growth rate
#define DivThreshold(x) func(3,x/X_SCALE) 	// position-dependent cell division threshold
#define L_FIN  2.03			// anticipated final filament length 
#define TH_RW  0.1925		// color threshold 1: red to white
#define TH_WB  0.6550		// color threshold 2: white to blue
#endif

#if (Panel==3)
// Figure 2ghi: variable growth, smootly increasing threshold
#define GrowthFunc(x)  0.01*func(1,x/X_SCALE)	// position-dependent growth rate
#define DivThreshold(x) func(3,x/X_SCALE)		// position-depoendent cell division threshold
#define L_FIN  1.81		// anticipated final filament length 
#define TH_RW  0.1925	// color threshold 1: red to white
#define TH_WB  0.5785	// color threshold 2: white to blue
#endif

Lsystem: 1

Start: {n=0;}	// initialize step counter
StartEach: {n=n+1; distance = 0;}	// distance from the tip

derivation length: STEPS

Axiom: 
	// Concentration plot with axes
	[f(HEIGHT+0.10) [ +(90) ;(8) PlotMC(0)] [;(1) Axis(0.60)] [+(90) ;(1) Axis(L_FIN+0.0)]] 
	// Cell division plot
	[f(HEIGHT - 0.65) [ +(90) ;(9)  PlotDT(0)]]
	// Growth function plot
	[f(HEIGHT - 0.65) [ +(90) ;(10) PlotGF(0)] [;(1) Axis(0.65)] [+(90) ;(1) Axis(L_FIN+0.0)]] 
	// Background rectangle for the plot
	[&(90) f(0.1) ^(90) ;(15) f(-0.78) {g(-0.20) -(90) f(0.05) +(90) f(HEIGHT+0.24) +(90) f(L_FIN+0.1) +(90) f(HEIGHT+0.25)}]
	// Filament development plot
	[f(HEIGHT-0.78) V(0) +(90) W(0) C(L_INIT,0.05) W(0)]

// Cell C(s,x) has parameters s: length, and x: distance from the tip of the filament
// Cell grows over time step dt; positional information propagates.
C(s,x) : 1 {x=distance; s=s*(e^(GrowthFunc(x)*dt)); distance=distance+s;} --> C(s, x)

// Cell wall aging
W(age) : 1 --> W(age+0.06)

// Vertical cell displacement in continuous plot
V(h) : n<STEPS-1 --> V(h-dhdt*dt)

// Vertical cell displacement in the last stage
V(h) --> V(h-0.1)

decomposition

// Cell divides upon reaching size threshold
C(s,x) : s>DivThreshold(x) --> C(s/2,x) W(0) C(s/2,x+s/2)

homomorphism:
maximum depth: 10000
// Definition of graphical representation

// Plot acordinate axis
Axis(x) --> F(x){.[-(160)f(0.05)][+(160)f(0.05)]} 

// Plot morphogen concentration function
PlotMC(x) : (x < L_FIN) && (n>STEPS-1) --> [-(90) f(MorphConc(x)) @c(0.010) ] f(0.001) PlotMC(x+0.001)
PlotMC(x) --> *

// Plot division threshold function
PlotDT(x) : x < L_FIN --> [-(90) f(DivThreshold(x)) @c(0.010) ] f(0.001) PlotDT(x+0.001)
PlotDT(x) --> *

// Plot growth function
PlotGF(x) : x < L_FIN --> [-(90) f(150*GrowthFunc(x)) @c(0.010) ] f(0.001) PlotGF(x+0.001)
PlotGF(x) --> *

// Displace plots of consecutive stages vertically 
V(h) --> f(h)

// Display cells in all stages butlast as lines; use French flag cell coloring
C(s,x) : n < STEPS-1 && x<TH_RW --> ;(5) F(s)
C(s,x) : n < STEPS-1 && x<TH_WB --> ;(6) F(s)
C(s,x) : n < STEPS-1  --> ;(7) F(s)
 
// Draw walls in all stages but last as dots; shared interior walls are wider than non-shared left and right borders
C(s,x) < W(age) > C(s,x) : n<STEPS-1 --> [;(255) ^(90)f(1)&(90) @c(0.01*(0.75+age/24))]
W(age) : n < STEPS-1 --> [;(1) ^(90)f(1)&(90) @c(0.005*(0.75+age/24))]

// Draw cells in the last stage; use French flag colloring
W(agel) < C(s,x) > W(ager) : 1 {l=s; w=0.1; bl=0.005*(1+agel/32); br=0.005*(1+ager/32); bw=0.01; r=0.015; 
 	 c=7; if (x<TH_WB) {c=6;} if (x<TH_RW) {c=5;} }-->
	 ;(1) R(l,w,r) [f(-l) ^ f(0.1) & ;(c) f(bl) R(l-(bl+br), w-2*bw, r-bw/2)] f(0.000)

// Cell boundary
R(l,w,r) : 1 {h = 2*r*1.41421;} --> [ !(1) - f((w-h)/2) |
	   { F(w-h) -(45) F(r) [- f(r) @c(h)] F(r) -(45)
	     F(l-h) -(45) F(r) [- f(r) @c(h)] F(r) -(45)
	     F(w-h) -(45) F(r) [- f(r) @c(h)] F(r) -(45)
	     F(l-h) -(45) F(r) [- f(r) @c(h)] F(r) -(45) }  ] f(l)
endlsystem
